#include <QX11Info>
#include <QColor>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <selinux/selinux.h>

bool __altselinux_enabled = false;

#include "altselinux.h"

bool altselinux_check_selinux()
{
	if (is_selinux_mls_enabled() <= 0) return false;
	struct stat stat_file;
	if (stat(ALT_SELINUX_SEAPPLET, &stat_file) != 0 || !(stat_file.st_mode & S_IXUSR)) return false;
	return true;
}

static char* _exec_write_read(char *cmd, char **arg, char *in, int in_size)
{
#define UTIL_BUF_SIZE	1024
	int pipe_in[2];
	int pipe_out[2];
	if (pipe(pipe_in) == -1) return NULL;
	if (pipe(pipe_out) == -1) return NULL;
	pid_t pid = fork();
	if (pid > 0)
	{
		close(pipe_in[0]);
		close(pipe_out[1]);
		char buf[UTIL_BUF_SIZE];
		char *result = (char*)malloc(1);
		int len, size = 0;
		if (in != NULL)
		{
			if (write(pipe_in[1], in, in_size) == -1)
			{
				close(pipe_in[1]);
				close(pipe_out[0]);
				free(result);
				return NULL;
			}
		}
		close(pipe_in[1]);
		while ((len = read(pipe_out[0], buf, UTIL_BUF_SIZE)) > 0)
		{
			result = (char*)realloc(result, size + len + 1);
			if (result == NULL)
			{
				close(pipe_out[0]);
				return NULL;
			}
			memcpy(result + size, buf, len);
			size += len;
		}
		close(pipe_out[0]);
		int status;
		if (wait(&status) == -1 || (!WIFEXITED(status) || (WEXITSTATUS(status) != 0)))
		{
			free(result);
			return NULL;
		}
		result[size] = 0;
		return result;
	}
	else if (pid == 0)
	{
		dup2(pipe_in[0], STDIN_FILENO);
		dup2(pipe_out[1], STDOUT_FILENO);
		close(pipe_in[0]);
		close(pipe_out[1]);
		execv(cmd, arg);
		_exit(1);
	}
	return NULL;
}

static char* _alt_get_context_color(char *context)
{
	char *argv[4];
	argv[0] = ALT_SELINUX_SEAPPLET;
	argv[1] = "color";
	argv[2] = context;
	argv[3] = NULL;
	return _exec_write_read(argv[0], argv, NULL, 0);
}

static char* _get_context(Window win)
{
	Atom type_ret;
	int format_ret;
	unsigned long items_ret;
	unsigned long after_ret;
	unsigned char *prop_data = 0;
	if (XGetWindowProperty(QX11Info::display(), win, XInternAtom(QX11Info::display(), "_SELINUX_CONTEXT", True), 0, (~0L), False, AnyPropertyType, &type_ret, &format_ret, &items_ret, &after_ret, &prop_data) != Success) return 0;
	return (char*)prop_data;
}

QColor altselinux_get_color(Window win)
{
	QColor result = QColor(-1, -1, -1);
	if (__altselinux_enabled == true)
	{
		char *context = _get_context(win);
		if (context != NULL)
		{
			char *str_color = _alt_get_context_color(context);
			if (str_color != NULL)
			{
				int R = 0, G = 0, B = 0;
				sscanf(str_color, "%d %d %d", &R, &G, &B);
				result = QColor(R, G, B);
				free(str_color);
			}
			XFree(context);
		}
	}
	return result;
}
